/* VIDEOCHATSCHOOL
 * =====================================================
 * Last updated: July 22, 2007.
 * 
 * Assumes that SCHOOL is the server.
 * 
 */


using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace VideoChat
{
    public partial class VideoChatSchool : Form
    {
        // constants
        const int AWARENESS_INTERVAL = 3000;
        const int PROXIMITY_MIN_TIMES = 5;
        const int RECORDING_LIGHT_INTERVAL = 500;
        const int CHECK_RFID_INTERVAL = 1000;
        const int CAMERA_AWAY_POSITION = 180;
        const int CAMERA_TOWARD_POSITION = 0;

        // variable declarations       
        bool present, twoWayTransmitting, cameraCapturing, blinkingRecordState, checkRFID;
        string visitorRFIDName;
        Timer awarenessTimer;
        Timer recordingLightTimer;
        int proxCount;
        EasyImages.CameraClient wc;
        GroupLab.GTNetworking.SharedDictionary SD;
        GroupLab.GTNetworking.Subscription imageSubscription;
        GroupLab.GTNetworking.Subscription pingSubscription;
        GroupLab.GTNetworking.Subscription isHereSubscription;
        GroupLab.GTNetworking.Subscription callerIDSubscription;
        GroupLab.GTNetworking.Subscription cameraCapturingSubscription;

        /* Thread handling: Need this block of code to make sure code runs from the main windows form, and not any other thread */
        public delegate void SetControlImage(PictureBox icontrol, Bitmap image); // delegate: pointer to a function
        public void RecievedImageToControl(PictureBox icontrol, Image image)
        {
            /* Control.InvokeRequired -- gets a value indicating whether the caller must call an invoke method when making method 
             * calls to the control because the caller is on a different thread than the one the control was created on. 
             * Control.BeginInvoke() -- executes a delegate asynchronously on the thread that the control's underlying handle was created on.
             */
            if (icontrol.InvokeRequired)
            {
                SetControlImage theDelegate = new SetControlImage(RecievedImageToControl);
                this.BeginInvoke(theDelegate, new object[] { icontrol, image });
            }
            else
                icontrol.Image = image;
        }//RecievedImageToControl

        public VideoChatSchool()
        {

            Control.CheckForIllegalCrossThreadCalls = false;

            InitializeComponent();

            // Set up timers
            awarenessTimer = new Timer();       // awareness Timer: sends frames using the specified interval so as to keep basic awareness
            awarenessTimer.Interval = AWARENESS_INTERVAL;
            recordingLightTimer = new Timer();   // blinking light Timer: blinks if awareness state is on
            recordingLightTimer.Interval = RECORDING_LIGHT_INTERVAL;

            // variable initializations
            wc = new EasyImages.CameraClient("DefaultCamera");
            wc.ReceivedFrame += new EasyImages.CameraClient.ReceivedFrameEventHandler(wc_ReceivedFrame);
            wc.Start();
            present = false;
            checkRFID = false;
            blinkingRecordState = true;
            twoWayTransmitting = false;
            cameraCapturing = true;
            proxCount = 0;
            visitorRFIDName = "--";

            // shared dictionary and subscription declarations
            SD = new GroupLab.GTNetworking.SharedDictionary();
            pingSubscription = new GroupLab.GTNetworking.Subscription();
            imageSubscription = new GroupLab.GTNetworking.Subscription();
            isHereSubscription = new GroupLab.GTNetworking.Subscription();
            callerIDSubscription = new GroupLab.GTNetworking.Subscription();
            cameraCapturingSubscription = new GroupLab.GTNetworking.Subscription();

            // Automatically connect to shared dictionary using localhost as the server (school: server)
            openConnection();

            // form settings
            this.playButton.Enabled = false;
            this.stopButton.Enabled = true;
            this.recordingLabel.ForeColor = Color.Goldenrod;
            this.visitorTextBox.Text = visitorRFIDName;
            if (this.interfaceKit1.Attached)                    // if interface kit and sensors are attached, then do not enable the here/not here buttons
            {
                this.hereRadiobutton.Enabled = false;
                this.notHereRadiobutton.Enabled = false;
            }
            else
            {
                this.hereRadiobutton.Enabled = true;
                this.notHereRadiobutton.Enabled = true;
            }
            this.notHereRadiobutton.Checked = true; // set default for not here 

            if (this.rfid1.Attached)
            {
                this.label1.Enabled = false;
                this.visitorTextBox.Enabled = false;                
            }
            else
            {
                if (this.notHereRadiobutton.Checked)// since user is not here (default), disable the visitor textbox
                {
                    this.label1.Enabled = false;
                    this.visitorTextBox.Enabled = false;                
                }
                else
                {
                    this.label1.Enabled = true;
                    this.visitorTextBox.Enabled = true;                
                }
            }
       
        }//VideoChatSchool (form)

        private void awarenessTimer_Tick(object sender, EventArgs e)
        {
            if (this.twoWayTransmitting) return; //if we are transmitting video two ways b/c both people are there, we don't want to use the awarenessTimer (send at normal frame rate)

            if (!this.cameraCapturing) return;

            if (this.SD.Status == GroupLab.GTNetworking.ConnectionStatus.Opened)
            {
                if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server) // Server (SCHOOL)
                {
                    // Get normal frame to send over network using videoDisplay
                    this.SD["/ServerImage"] = new EasyImages.JPEG(wc.GetFrame(), 39);

                    // Display what is actually sent over the network
                    if (this.SD.Contains("/ServerImage"))
                        RecievedImageToControl(this.actualDisplay, (this.SD["/ServerImage"] as EasyImages.JPEG).GetImage()); // Display what is actually sent over network
                }
            }//if

        }//awarenessTimer_Tick

        private void recordingLightTimer_Tick(object sender, EventArgs e)
        {
            if (blinkingRecordState)
            {
                if (this.interfaceKit1 != null && this.interfaceKit1.Attached)
                    this.interfaceKit1.Outputs[2].State = !this.interfaceKit1.Outputs[2].State;
            }
        }//recordingLightTimer_Tick

        private void wc_ReceivedFrame(object sender, EasyImages.CameraEventArgs e)
        {
            RecievedImageToControl(this.mirrorDisplay, e.Bitmap); //this.MirrorDisplay.Image = e.Bitmap;

        }//wc_ReceivedFrame

        private void openConnection()
        {
            this.SD.Url = "tcp://localhost:TwinnedMediaSpace";
            this.SD.Opened += new EventHandler(SD_Opened);
            this.SD.Closed += new EventHandler(SD_Closed);
            this.SD.Open();

            // Sensor changes
            if (this.interfaceKit1 != null && this.interfaceKit1.Attached)
                this.interfaceKit1.SensorChange += new GroupLab.SharedPhidgets.IndexSensorChangeEventHandler(this.interfaceKit1_SensorChange);

            // Timers
            awarenessTimer.Tick += new EventHandler(awarenessTimer_Tick);
            awarenessTimer.Start();
            recordingLightTimer.Tick += new EventHandler(recordingLightTimer_Tick);
            recordingLightTimer.Start();

        }//openConnection

        private void SD_Opened(object sender, EventArgs e)
        {
           // Connects the shared dictionary to the SQL server 
            // after this, SD is connected to the server and ready to use the persistent features
            SD.ConnectToSQLServer("server=SAULNU\\SQLEXPRESS;" +
                                       "Trusted_Connection=yes;" +
                                       "database=master;" +
                                       "connection timeout=30");
            // following line loads stored values back into the shared dictionary
            SD.LoadStoredValues();


            // imageSubscription
            this.imageSubscription.BeginInit();
            this.imageSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                this.imageSubscription.Pattern = "/ClientImage";
            this.imageSubscription.Notified += new GroupLab.GTNetworking.SubscriptionEventHandler(imageSubscription_Notified);
            this.imageSubscription.EndInit();

            // pingSubscription
            this.pingSubscription.BeginInit();
            this.pingSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                this.pingSubscription.Pattern = "/ClientPing";
            this.pingSubscription.Notified += new GroupLab.GTNetworking.SubscriptionEventHandler(pingSubscription_Notified);
            this.pingSubscription.EndInit();

            // send the first frame
            //takeSnap();

            // isHereSubscription
            this.isHereSubscription.BeginInit();
            this.isHereSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
            {
                this.isHereSubscription.Pattern = "/ClientIsHere";
                this.SD["/ClientIsHere"] = false;
                this.SD["/ServerIsHere"] = false;
            }
            this.isHereSubscription.Notified += new GroupLab.GTNetworking.SubscriptionEventHandler(isHereSubscription_Notified);
            this.isHereSubscription.EndInit();

            // callerIDSubscription
            this.callerIDSubscription.BeginInit();
            this.callerIDSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                this.callerIDSubscription.Pattern = "/ClientCallerID";
            this.callerIDSubscription.Notified += new GroupLab.GTNetworking.SubscriptionEventHandler(callerIDSubscription_Notified);
            this.callerIDSubscription.EndInit();

            // cameraCapturingSubscription
            this.cameraCapturingSubscription.BeginInit();
            this.cameraCapturingSubscription.Dictionary = this.SD;
            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
            {
                this.cameraCapturingSubscription.Pattern = "/ClientCameraCapturing";
                this.SD["/ClientCameraCapturing"] = false;
                this.SD["/ServerCameraCapturing"] = false;
            }
            this.cameraCapturingSubscription.Notified += new GroupLab.GTNetworking.SubscriptionEventHandler(cameraCapturingSubscription_Notified);
            this.cameraCapturingSubscription.EndInit();

            // set initial values for cameraCapturingSubscription
            if (this.cameraCapturing)
                this.SD["ServerCameraCapturing"] = true;
            else
                this.SD["ServerCameraCapturing"] = false;

        }// SD_Opened

        private void SD_Closed(object sender, System.EventArgs args)
        {
            if (this.interfaceKit1 != null && this.interfaceKit1.Attached)
                this.interfaceKit1.Outputs[2].State = false; // recording light            

        }//SD_Closed

        private void takeSnap()
        {
            if (this.SD.Status == GroupLab.GTNetworking.ConnectionStatus.Opened)
            {
                if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                {
                    if (cameraCapturing)
                        this.SD["/ServerImage"] = new EasyImages.JPEG(wc.GetFrame(), 39);
                }
            }//if
        }//takeSnap

        private void imageSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)
        {
            EasyImages.JPEG j = e.Value as EasyImages.JPEG;
            if (j != null)
            {
                RecievedImageToControl(this.videoDisplay, j.GetImage());  //this.VideoDisplay.Image = j.GetImage();                    
            }

            if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
            {         
                // Display what is actually sent over network
                if (this.SD.Contains("/ServerImage"))
                    RecievedImageToControl(this.actualDisplay, (this.SD["/ServerImage"] as EasyImages.JPEG).GetImage());

                // indicate that image has been recieved and can now get the next one (stop and wait)
                this.SD["/ServerPing"] = "ping!";
            }
        }//imageSubscription_Notified

        private void pingSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)
        {
            if (twoWayTransmitting) takeSnap();
        }//pingSubscription_Notified

        private void callerIDSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)
        {
            //throw new Exception("The method or operation is not implemented.");
        }//callerIDSubscription_Notified

        private void cameraCapturingSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)
        {
         //   throw new Exception("The method or operation is not implemented.");
        }//cameraCapturingSubscription_Notified

        private void checkPresence()
        {
            bool otherPresent = false;

            if (SD.ConnectionType.Equals(GroupLab.GTNetworking.ConnectionType.Server))
            {
                otherPresent = (bool)this.SD["/ClientIsHere"];
                Console.WriteLine("otherPresent: " + otherPresent);
            }

            // case 1: This is here, other isn't
            if (present && !otherPresent)
            {
                Console.WriteLine("State 1: this here, other isn't");
                this.SD["/WaitingOnHome"] = true;
                twoWayTransmitting = false;                
                setRecordingStateLights("blink");
            }
            // case 2: This isn't here, other is  
            else if (!present && otherPresent)
            {
                Console.WriteLine("State 2: This not here, other is here");
                twoWayTransmitting = false;                
                setRecordingStateLights("blink");
            }
            // case 3: both there
            else if (present && otherPresent)
            {
                Console.WriteLine("State 3: Both here");
                if (cameraCapturing && (bool)this.SD["/ClientCameraCapturing"])    // if both here and both cameras are capturing
                {
                    setRecordingStateLights("solid");
                    twoWayTransmitting = true;
                    takeSnap();
                }
                else if (cameraCapturing && !(bool)this.SD["/ClientCameraCapturing"])   // both here, this camera is capturing but other is not
                {
                    setRecordingStateLights("blink");
                    twoWayTransmitting = false;                    
                }
                else if (!cameraCapturing && (bool)this.SD["/ClientCameraCapturing"])   // both here, this camera not capturing but other is
                {
                    setRecordingStateLights("none");
                    twoWayTransmitting = false;
                }
                else if (!cameraCapturing && !(bool)this.SD["/ClientCameraCapturing"])   // both here, but both camera's not capturing
                {
                    setRecordingStateLights("none");
                    twoWayTransmitting = false;
                }
            }
            // case 4: both gone
            else
            {
                Console.WriteLine("State 4: Both not here");
                twoWayTransmitting = false;                
                setRecordingStateLights("blink");
            }
        }//checkPresence

        private void isHereSubscription_Notified(object sender, GroupLab.GTNetworking.SubscriptionEventArgs e)
        {
            checkPresence();
        }//isHereSubscription_Notified

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            // if user chooses to close the form manually, do so in an elegant fashion (stop camera and close dictionary)
            wc.Stop();
            setRecordingStateLights("none");
            if (SD.Status == GroupLab.GTNetworking.ConnectionStatus.Opened)
            {
                SD.Close();
            }
        }//Form1_FormClosing

        private void setPresence(bool pres)
        {
            // set the isHere subscription
            if (pres)
            {
                present = true;
                if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                {
                    this.SD["/ServerIsHere"] = true;
                }
            }
            else
            {
                present = false;
                if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                {
                    this.SD["/ServerIsHere"] = false;
                    this.SD["/WaitingOnHome"] = false;
                }
            }
            
            // set the visitor's name
            if (this.rfid1.Attached)
            {
                if (present)
                {
                    if (visitorRFIDName.Equals("--"))  // first time visitor has sat down, RFID name not known yet 
                    {
                        visitorRFIDName = "Visitor"; // set to visitor by default first
                        this.visitorTextBox.Text = visitorRFIDName;
                        this.SD["/ServerCallerID"] = visitorRFIDName;
                        checkRFID = true;
                    }
                    else if (!visitorRFIDName.Equals("Visitor") && !visitorRFIDName.Equals("--")) // RFID name known, no need to check anymore
                    {                       
                        checkRFID = false;
                    }
                    else if (visitorRFIDName.Equals("Visitor"))
                    {                
                        checkRFID = true;
                    }
                }
                else //not present
                {

                    // if user has gotten up, clear the name (if there was a name), and do not check                    
                    visitorRFIDName = "--";
                    this.SD["/ServerCallerID"] = visitorRFIDName;
                    this.visitorTextBox.Text = visitorRFIDName;
                    checkRFID = false;
                }
            }//if rfid is attached

        }//setPresence

        private void interfaceKit1_SensorChange(object sender, GroupLab.SharedPhidgets.IndexSensorChangeEventArgs e)
        {
            if (SD.Status != GroupLab.GTNetworking.ConnectionStatus.Opened) return;

            // Light sensor on chair to detect presence
            if (e.Index == 0)
            {
                if (e.Value < 100)              // sitting in chair
                {
                    this.hereRadiobutton.Checked = true;
                }
                else if (e.Value >= 100)        // not sitting in chair
                {
                    this.notHereRadiobutton.Checked = true;
                }
            }

            // Proximity sensor attached to the camera to turn camera on/off
            if (e.Index == 1)
            {
                if (e.Value >= 200 && e.Value <= 250)       // turn camera on or off if proximity within this value
                {
                    proxCount++;
                    if (proxCount >= PROXIMITY_MIN_TIMES)
                    {
                        proxCount = 0;
                        if (this.cameraCapturing)
                            setCamera(0);
                        else
                            setCamera(1);
                    }
                }//if
                else
                    proxCount = 0;
            }//if proximity sensor  

        }//interfaceKit1_SensorChange

        private void setRecordingStateLights(string state)
        {
            switch (state)
            {
                case "solid":
                    Console.WriteLine("Solid both transmitting");
                    this.recordingLabel.ForeColor = Color.Red;
                    blinkingRecordState = false;
                    if (this.interfaceKit1 != null && this.interfaceKit1.Attached)
                        this.interfaceKit1.Outputs[2].State = true;
                    break;
                case "blink":
                    Console.WriteLine("Blinking awareness state");
                    this.recordingLabel.ForeColor = Color.Goldenrod;
                    blinkingRecordState = true;
                    break;
                case "none":
                    Console.WriteLine("not capturing anything");
                    this.recordingLabel.ForeColor = Color.LightGray;
                    blinkingRecordState = false;
                    if (this.interfaceKit1 != null && this.interfaceKit1.Attached)
                        this.interfaceKit1.Outputs[2].State = false;
                    break;
                default:
                    Console.WriteLine("default recording state lights");
                    break;

            }//switch            

        }//setRecordingStateLights

        private void playButton_Click(object sender, EventArgs e)
        {
            setCamera(1); // start camera                            
        }//playButton_Click

        private void stopButton_Click(object sender, EventArgs e)
        {
            setCamera(0); // stop camera
        }//stopButton_Click

        private void setCamera(int action)
        {
            switch (action)
            {
                case 0:     // Stop camera                    
                    wc.Stop();

                    this.playButton.Enabled = true;
                    this.stopButton.Enabled = false;

                    // use servo to change the camera angle away from person
                    if (this.servo1.Attached)
                        this.servo1.Motors[0].Position = CAMERA_AWAY_POSITION;

                    awarenessTimer.Stop();
                    this.twoWayTransmitting = false; // set to false because in ping_Notified, wont' be calling takeSnap to get the next frame (only send the stopped image)
                    this.cameraCapturing = false;
                   
                    if (this.hereRadiobutton.Checked)
                        setPresence(true);
                    else
                        setPresence(false);
                    checkPresence();
     
                    // show the stopped image in the actual display
                    Image stoppedImage = Image.FromFile("stopped.jpg");
                    RecievedImageToControl(this.actualDisplay, stoppedImage);

                    // show stopped image in video display (depending on the connection type)
                    if (this.SD.Status == GroupLab.GTNetworking.ConnectionStatus.Opened)
                    {
                        if (this.SD.ConnectionType == GroupLab.GTNetworking.ConnectionType.Server)
                            this.SD["/ServerImage"] = new EasyImages.JPEG(stoppedImage, 39);
                    }
                    break;

                case 1:     // Start camera
                    wc.Start();

                    this.playButton.Enabled = false;
                    this.stopButton.Enabled = true;

                    // use servo to change the camera angle to face towards person
                    if (this.servo1.Attached)
                        this.servo1.Motors[0].Position = CAMERA_TOWARD_POSITION;
                    
                    this.cameraCapturing = true;                    
                    if (this.hereRadiobutton.Checked)
                        setPresence(true);
                    else
                        setPresence(false);
                    checkPresence();
                    awarenessTimer.Start();
                    this.awarenessTimer_Tick(null, null);
                    break;

                default:
                    Console.WriteLine("setCamera default switch statement");
                    break;
            }//switch

            if (this.cameraCapturing)
                this.SD["ServerCameraCapturing"] = true;
            else
                this.SD["ServerCameraCapturing"] = false;
            checkPresence();
        }//setCamera

        private void rfid1_Tag(object sender, GroupLab.SharedPhidgets.RFIDTagEventArgs e)
        {
            if (checkRFID)
            {
                Console.WriteLine("Checking RFID");
                if (SD.Contains("/TwinMediaRFIDS/" + e.Tag))
                {
                    visitorRFIDName = SD["/TwinMediaRFIDS/" + e.Tag].ToString();
                }
                else
                {
                    visitorRFIDName = "Visitor";   // case where user has an RFID tag, but this tag does not exist in the database yet.
                }
                this.visitorTextBox.Text = visitorRFIDName;
                this.SD["/ServerCallerID"] = visitorRFIDName;
            }

        }//rfid1_Tag

        private void visitorTextBox_TextChanged(object sender, EventArgs e)//notHereRadioButton_CheckedChanged
        {
            if (this.visitorTextBox.Text.Length == 0)
                this.visitorTextBox.Text = "--";
               
            this.SD["/ServerCallerID"] = this.visitorTextBox.Text;                        
    
        }//visitorTextBox_TextChanged

        private void presenceRadioGroup_CheckedChanged(object sender, EventArgs e)
        {
            // here/not here options
            RadioButton radButton = (RadioButton)sender;

            if (radButton.Text.Equals("Here") && radButton.Checked)
            {                
                // bold only the checked button                                
                this.hereRadiobutton.Font = new Font(this.hereRadiobutton.Font, FontStyle.Bold);
                this.notHereRadiobutton.Font = new Font(this.notHereRadiobutton.Font, FontStyle.Regular);
                
                // set presence variables
                setPresence(true);
                checkPresence();

                // set identity variables
                this.label1.Enabled = true;
                this.visitorTextBox.Enabled = true;
            }
            else if (radButton.Text.Equals("Not Here") && radButton.Checked)
            {             
                // bold only the checked button            
                this.notHereRadiobutton.Font = new Font(this.notHereRadiobutton.Font, FontStyle.Bold);
                this.hereRadiobutton.Font = new Font(this.hereRadiobutton.Font, FontStyle.Regular);

                // set presence variables
                setPresence(false);
                checkPresence();

                // set identity variables
                this.visitorTextBox.Text = "--";
                this.label1.Enabled = false;
                this.visitorTextBox.Enabled = false;
                
            }                        

        }//presenceRadioGroup_CheckedChanged


    }// partial class Form1
}// namespace VideoChat